#include <alloc.h>
#include <dir.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "password.h"
#include "define.h"
#include "tools.h"
#include "users.h"
#include "buffers.h"
#include "node.h"
extern "C" {
#include "md2.h"
};

extern	tUserCfg * pUserCfg;

//--------------------------------------------------------------------------
//------ Envoyer la cle  l'utilisateur -----------------------------------
//--------------------------------------------------------------------------
//La retourne TRUE si la cle a pu tre transmise, et FALSE sinon
int PASSWORD_sendKey(int StreamNum, char * CallSign)
{
  char	sPass[256];
  char	sKey[300];
  char	sNodeCall[11];
  int	index;
  int	iPos;
  time_t t;

  //Accder  la config de l'utilisateur
  USERS_getRecord (StreamNum);

  //Quel est le password ?
  if( ! PASSWORD_getPass(CallSign, sPass) )
    return FALSE;	/*Password introuvable*/

  //Initialiser le gnrateur de nombres alatoires
  srand((unsigned) time(&t));

  //Pour respecter le format standard ...
  BUFFERS_printBuff(StreamNum, OUT, "%s> ", NODE_getNodeCall(0, 0, sNodeCall));

  //Lire 5 caractres alatoirement
  for(index = 0; index < 5; index++)
  {
    //Gnrer un nombre alatoire de position dans la chaine
    iPos = rand() % strlen(sPass);

    //Envoyer la position  l'utilisateur et mmoriser le caractre
    BUFFERS_printBuff(StreamNum, OUT, "%d ", iPos + 1);
    pUserCfg->sPassword[index] = sPass[iPos];
  }/*End FOR*/

  pUserCfg->sPassword[5] = SNULL;

  //Envoyer, calculer et mmoriser la chaine MD2
  BUFFERS_printBuff(StreamNum, OUT, "[%010ld]\n", t);
  sprintf(sKey, "%010ld%s", t, sPass);
  MD2String((unsigned char*)pUserCfg->sMD2, (unsigned char*) sKey);

  //Updater la config utilisateur
  USERS_updateLocalConfig(StreamNum);

  return TRUE;
}

//--------------------------------------------------------------------------
//------ Acceder a la chaine -----------------------------------------------
//--------------------------------------------------------------------------
int PASSWORD_getPass(char * pszCallSign, char * pszPass)
{
  FILE * fPtr;
  char * pPtr;
  char   szUserCall[16];
  char	 szLine[256];


  /*Ouvrir le fichier password*/
  fPtr = fopen(PASSWORD_FILE, "rt");

  if( ! fPtr )
  {
    perror("fopen in PASSWORD_sendKey");
    return FALSE;	/*Le fichier n'a pas pu tre ouvert*/
  }/*End IF*/

  /*Chercher l'indicatif de l'utilisateur dans le fichier*/
  strcpy(szUserCall, pszCallSign);
  TOOLS_removeSsid (szUserCall);
  for(;;)
  {
    /*Lire une ligne dans le fichier*/
    if( ! fgets(szLine, 255, fPtr) )
    {
      fclose (fPtr);
      return FALSE;	/*Fin de fichier atteinte / indicatif non trouve*/
    }

    TOOLS_removeNR (szLine);

    /*Isoler l'indicatif dans la chaine lue ...*/
    pPtr = szLine;
    while( *pPtr != ' ' && *pPtr != SNULL )
      pPtr++;

    if( *pPtr == SNULL )
    {
      fclose (fPtr);
      return FALSE;	/*Mot de passe inexistant ...*/
    }

    *pPtr = SNULL;
    strupr (szLine);		/*Pour tre en rapport avec szUserCall*/
    TOOLS_removeSsid (szLine);  /*Normallement inutile ...*/

    /*Est-ce le bon indicatif ?*/
    if( ! strcmp(szUserCall, szLine) )
    {
      /*OUI - Passer les espaces qui separent l'indicatif du password*/
      pPtr++;
      while( *pPtr == ' ' && *pPtr != SNULL )
	pPtr++;

      if( *pPtr == SNULL )
      {
	fclose (fPtr);
	return FALSE;		/*Mot de passe inexistant ...*/
      }

      /*Tout est OK, quitter la boucle*/
      strcpy(pszPass, pPtr);
      fclose (fPtr);
      return TRUE;
    }/*End IF*/
  }/*End LOOP*/
}

//--------------------------------------------------------------------------
//------ Vrifier le password pour un utilisateur --------------------------
//--------------------------------------------------------------------------
int PASSWORD_verify(int StreamNum, char * Pass)
{
  //Accder  la config de l'utilisateur
  USERS_getRecord(StreamNum);

  //Verifier la concordance
  if( ! strcmp(pUserCfg->sMD2, Pass) )
    return TRUE;	//C'est bon, retourner TRUE
  else if( ! strcmp(pUserCfg->sPassword, Pass) )
    return TRUE; 	//C'est bon, retourner TRUE
  else
    return FALSE;	//Faux !
}
