#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "purgdate.h"
#include "purgann.h"

/* This program is intended to purge SYSTEM\BIN\ANNOUNCE.BIN */

/* PURGEANN [p=AnnPath] [d=#n_days] [v]

   where : - [p=AnnPath] specifies the directory in where announce.bin file
	     is located.
	     The default directory is ..\SYSTEM\BIN.

	   - [d=#n_days] sets the number of day (#n_days) after which an
	     annoucement will be deleted.
	     By default, this value is set to 30.

	   - [v] prints the version number.

   Example : PURGANN p=C:\DXNET\SYSTEM\BIN d=7
	or : PURGANN d=15

*/

/*---------------------------------------------------------------------------
  ------ main function ------------------------------------------------------
  -------------------------------------------------------------------------*/
int main(int Argc, char * Argv[])
{
	int	index;
	int	nSpotNum    = 0;
	int	nDelSpotNum = 0;
	int     nDelDays   = 30;
	char	szBinPath[1024];
	char	szBin[256];
	char	szOld[256];
	FILE *	fBinFile;
	FILE *	fOldFile;
	struct AnnounceStruct tAnnounce;

#ifdef LINUX
	strcpy (szBinPath,  "system/bin/");
#else
	strcpy (szBinPath,  "system\\bin\\");
#endif

	/* Get the arguments */
	for(index = 1; index < Argc; index++)
	{
		switch( Argv[index][0] )
		{
			case 'd' :
			case 'D' :
				if( Argv[index][1] == '=' )
				nDelDays = atoi (Argv[index] + 2);
				break;

			case 'p' :
			case 'P' :
				if( Argv[index][1] == '=' )
				strcpy (szBinPath, Argv[index] + 2);
				break;

			case 'v' :
			case 'V' :
				printf("PURGANN v 1.2\n");
				break;
		}/* End SWITCH */
	}/* End FOR */

	/* Check arguments */
	if( nDelDays <= 0 )
		nDelDays = 1;

	/* Prepare the file name */
#ifdef LINUX
	if( szBin[strlen (szBin)] == '/' )
		sprintf (szBin, "%s%s", szBinPath, "announce.bin");
	else
		sprintf (szBin, "%s%s", szBinPath, "/announce.bin");
#else
	if( szBin[strlen (szBin)] == '\\' )
		sprintf (szBin, "%s%s", szBinPath, "announce.bin");
	else
		sprintf (szBin, "%s%s", szBinPath, "\\announce.bin");
#endif
	strcpy (szOld, szBin);
	strcpy (szOld + strlen (szOld) - 4, ".old");

	/* Create a bak file */
	/* Open the files */
	fBinFile = fopen (szBin, "rb");
	if( ! fBinFile )	/* Error opening the file */
	{
		perror ("PURGANN -> fopen in MAIN (fBinFile 1)");
		return (1);
	}

	fOldFile = fopen (szOld, "w+b");
	if( ! fOldFile )	/* Error opening the file */
	{
		perror ("PURGANN -> fopen in MAIN (fOldFile 1)");
		return (2);
	}

	/* Copy announce.bin -> announce.old */
	printf ("Creating announce.old\n");
	for (;;)
	{
		if( ! fread (&tAnnounce, sizeof (tAnnounce), 1, fBinFile) )
			break;	/* EOF */
		fwrite (&tAnnounce, sizeof (tAnnounce), 1, fOldFile);
	}/* End FOR */

	fclose (fBinFile);
	fclose (fOldFile);

	fBinFile = fopen (szBin, "wb");
	if( ! fBinFile )	/* Error opening the file */
	{
		perror ("PURGANN -> fopen in MAIN (fBinOld 2)");
		return (3);
	}

	fOldFile = fopen (szOld, "rb");
	if( ! fOldFile )	/* Error opening the file */
	{
		perror ("PURGANN -> fopen in MAIN (fOldFile 2)");
		return (2);
	}

	for (;;)
	{
		char   szDate[16];
		char   szTime[16];
		time_t SpotTime;

		if( ! fread (&tAnnounce, sizeof (tAnnounce), 1, fOldFile) )
			break;	/* EOF */

		/* Is this announcement to be killed ? */
		DATE_dateTime_Long2Str (szDate, szTime, tAnnounce.lDateTime);
		SpotTime = DATE_hms (szDate);
		if( difftime (time (NULL), SpotTime) < (86400L * (double) nDelDays) )
		{
		  /* No, copy it */
		  fwrite(&tAnnounce, sizeof (tAnnounce), 1, fBinFile);
		  nSpotNum++;
		}
		else
		{
		  /* Yes, do not copy it */
		  nDelSpotNum++;
		} /* End IF */
	} /* End FOR  */

	fclose (fOldFile);
	fclose (fBinFile);

	printf ("PURGANN completed : %d announcements left, %d deleted.\n",
		nSpotNum, nDelSpotNum);

	return (0);
}
